"""
D3cipher Gateway — Multi-agent LockStock reverse proxy.

Run with: uvicorn lockstock_gateway:app --host 0.0.0.0 --port 4000
"""

from .gateway import D3cipherGateway

app = D3cipherGateway()

__all__ = ["D3cipherGateway", "app"]
